import argparse
import os
import sys
from typing import List

from shopping_list_generator.recipelocator import RecipeParsingError, read_and_parse_recipe
from shopping_list_generator.shoppinglist import ShoppingList


def main(argv: List[str]) -> int:
    parser = argparse.ArgumentParser()
    parser.add_argument("recipes", help="relative path of the recipe files")
    args = parser.parse_args(argv[1:])

    # Strip white space and append .txt to filename string if it does not end with .txt
    recipe_file_names = [x.strip() + '.txt' if not x.endswith('.txt') else x.strip()
                         for x in args.recipes.split(',')]
    shopping_list = ShoppingList()
    for recipe_file_name in recipe_file_names:
        try:
            recipe = read_and_parse_recipe(recipe_file_name)
            shopping_list.add_recipe(recipe)
        except IOError as err:
            print(f"Could not read the recipe file {recipe_file_name}: {err}")
            return 1
        except RecipeParsingError as err:
            print(f"Could not parse recipe {recipe_file_name}: {err}")
            return 1

    shopping_list_text = shopping_list.get_shopping_list_text()

    cwd = os.getcwd()
    shopping_list_path = os.path.join(cwd, "shoppinglist.txt")

    try:
        with open(shopping_list_path, 'w') as filetowrite:
            filetowrite.write(shopping_list_text)
    except IOError as err:
        print(f"Could not write to file {shopping_list_path}: {err}")
        return 1

    return 0


def cli_entrypoint():
    sys.exit(main(sys.argv))


if __name__ == "__main__":
    cli_entrypoint()




import os
from typing import Tuple
from shopping_list_generator.recipe import Recipe
from pathlib import Path


class RecipeParsingError(Exception):
    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


def read_and_parse_recipe(relative_recipe_path: str) -> Recipe:
    cwd = os.getcwd()
    recipe_path = os.path.join(cwd, relative_recipe_path)
    new_recipe_name = Path(recipe_path).stem
    new_recipe = Recipe(new_recipe_name)

    for line in open(recipe_path):
        if not line.strip():
            continue
        format_err_msg = f"""Formatting error in {recipe_path} on line '{line}'.
        Ingredient must be in style '1 tomato'.
        With each ingredient on a new line"""

        (ingredient_amount, ingredient_name) = parse_recipe_text_line(line, format_err_msg)
        new_recipe.add_ingredient(ingredient_name, ingredient_amount)

    return new_recipe


def parse_recipe_text_line(line: str, error_msg: str) -> Tuple[int, str]:
    formatted_line = line.strip().lower()
    line_split = formatted_line.split(sep=" ", maxsplit=1)

    if len(line_split) != 2:
        raise RecipeParsingError(error_msg)

    try:
        ingredient_num = int(line_split[0])
        ingredient_name = line_split[1].strip()
        return (ingredient_num, ingredient_name)
    except ValueError:
        raise RecipeParsingError(error_msg)





from dataclasses import dataclass
from typing import Counter, List
from shopping_list_generator.recipe import Recipe


@dataclass
class ShoppingList:
    _recipes: List[Recipe]

    def __init__(self):
        self._recipes = []

    def add_recipe(self, recipe: Recipe):
        self._recipes.append(recipe)

    def get_shopping_list_text(self) -> str:
        combined_ingredients = Counter()
        for recipe in self._recipes:
            combined_ingredients += recipe.get_ingredients()

        return "\n".join([f'{count} {name}' for (name, count) in combined_ingredients.items()])



from dataclasses import dataclass
from typing import Counter


@dataclass
class Recipe:
    recipe_name: str
    _ingredients: Counter

    def __init__(self, recipe_name: str):
        self.name = recipe_name
        self._ingredients = {}

    def add_ingredient(self, ingredient_name: str, amount: int):
        if ingredient_name in self._ingredients:
            self._ingredients[ingredient_name] += amount
        else:
            self._ingredients[ingredient_name] = amount

    def get_ingredients(self) -> Counter:
        return self._ingredients



import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "../README.md").read_text()

# This call to setup() does all the work
setup(
    name="shopping-list-generator",
    version="0.0.5",
    description="Generate a shopping list from recipe .txt files",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/CallumHoughton18/shopping-list-generator",
    author="Callum Houghton",
    author_email="callum.houghton13@hotmail.co.uk",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    packages=["shopping_list_generator"],
    include_package_data=True,
    install_requires=[],
    entry_points={
        "console_scripts": [
            "shopping-list-generator=shopping_list_generator.__main__:cli_entrypoint",
        ]
    },
)
